/*******************************************************************************
 * PTV xServer - Code Samples
 * 
 * PTV AG grants you a nonexclusive copyright license
 * to use all programmingcode examples from which you
 * can generate similar function tailored to your own
 * specific needs.
 * 
 * All sample code is provided by PTV AG for illustrative
 * purposes only. These examples have not been thoroughly
 * tested under all conditions. PTV AG, therefore, cannot
 * guarantee or imply reliability, serviceability, or
 * function of these programs.
 * 
 * All programs contained herein are provided to you
 * "AS IS" without any warranties of any kind. The implied
 * warranties of non-infringement, merchantability and
 * fitness for a particular purpose are expressly disclaimed.
 * 
 * (c) 2012 PTV AG, Karlsruhe Germany
 ******************************************************************************/
package samples.xmap.advanced;

/*
 * PTV xServer Code Samples
 * 
 * PTV xMap Server - how to calculate and display a route into a map.
 * 
 * Required servers: xMap, xRoute
 * 
 * (c) 2012 PTV AG
 */
import java.awt.HeadlessException;
import java.net.MalformedURLException;
import java.net.URL;
import java.rmi.RemoteException;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

import utils.XMapUtils;
import utils.XRouteUtils;

import com.ptvag.jabba.core.exception.SystemException;
import com.ptvag.jabba.core.exception.UnexpectedException;
import com.ptvag.jabba.service.baseservices.CallerContext;
import com.ptvag.jabba.service.baseservices.CallerContextProperty;
import com.ptvag.xserver.common.BoundingBox;
import com.ptvag.xserver.common.LineString;
import com.ptvag.xserver.common.PlainPoint;
import com.ptvag.xserver.common.Point;
import com.ptvag.xserver.common.XServiceException;
import com.ptvag.xserver.xmap.Color;
import com.ptvag.xserver.xmap.CustomLayer;
import com.ptvag.xserver.xmap.ImageFileFormat;
import com.ptvag.xserver.xmap.ImageInfo;
import com.ptvag.xserver.xmap.Layer;
import com.ptvag.xserver.xmap.LineOptions;
import com.ptvag.xserver.xmap.LinePartOptions;
import com.ptvag.xserver.xmap.Lines;
import com.ptvag.xserver.xmap.Map;
import com.ptvag.xserver.xmap.MapParams;
import com.ptvag.xserver.xmap.ObjectInfoType;
import com.ptvag.xserver.xmap.XMapException;
import com.ptvag.xserver.xmap.XMapRemoteInterface;
import com.ptvag.xserver.xroute.DetailLevel;
import com.ptvag.xserver.xroute.LinkType;
import com.ptvag.xserver.xroute.ResultListOptions;
import com.ptvag.xserver.xroute.Route;
import com.ptvag.xserver.xroute.WaypointDesc;
import com.ptvag.xserver.xroute.XRouteException;
import com.ptvag.xserver.xroute.XRouteRemoteInterface;

public class MappingWithRoute {
	public static void main(String[] args) {
		runSample();
	}

	private static void runSample() {
		// Step 1: initialize clients for xMap and xRoute. Use the utility classes to define the clients.
		final String XMAP_WS_URL = "http://localhost:50010/xmap/ws/XMap";
		XMapRemoteInterface xMapClient = XMapUtils.makeClient(XMAP_WS_URL);

		final String XROUTE_WS_URL = "http://localhost:50030/xroute/ws/XRoute";
		XRouteRemoteInterface xRouteClient = XRouteUtils
				.makeClient(XROUTE_WS_URL);

		try {
			// Step 3: calculate a route
			Route route = calculateRoute(xRouteClient);
			// Step 4: map the route
			displayMapOfRoute(xMapClient, route);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Calculates and it returns a route that is suitable for mapping.
	 * 
	 * It uses xRoute to calculate the route.
	 * 
	 * @param xRouteClient
	 *            an xRoute client
	 * @return a route (including a polygon -> suitable for mapping)
	 * @throws UnexpectedException
	 * @throws XRouteException
	 * @throws XServiceException
	 * @throws SystemException
	 * @throws RemoteException
	 */
	private static Route calculateRoute(XRouteRemoteInterface xRouteClient)
			throws UnexpectedException, XRouteException, XServiceException,
			SystemException, RemoteException {

		// Step 3.1: Specify the ResultListOptions
		ResultListOptions resultListOptions = new ResultListOptions();
		resultListOptions.setPolygon(true);
		resultListOptions.setTotalRectangle(true);
		resultListOptions.setDetailLevel(DetailLevel.STANDARD);

		// Step 3.2: Set WKB (well-known binary) as geometry format
		CallerContextProperty property = new CallerContextProperty(
				"ResponseGeometry", "WKB");
		CallerContext callerContext = new CallerContext();
		callerContext.setProperties(new CallerContextProperty[] { property });

		// Step 3.3: Get the way points from the getWaypoints() function
		final WaypointDesc[] waypointDescs = getWaypoints();

		// Step 3.4: Return object route as a polygon using calculateRoute() function
		Route route = xRouteClient.calculateRoute(waypointDescs, null, null,
				resultListOptions);

		return route;
	}

	/**
	 * Uses xMap to create a map of a route and displays it in a JFrame
	 * 
	 * @param xMapClient
	 *            an XMap WS client
	 * @param route
	 *            a route from xRoute (must include a polygon)
	 * @throws RemoteException
	 * @throws SystemException
	 * @throws XMapException
	 * @throws UnexpectedException
	 * @throws XServiceException
	 * @throws MalformedURLException
	 * @throws HeadlessException
	 */
	private static void displayMapOfRoute(XMapRemoteInterface xMapClient,
			Route route) throws RemoteException, SystemException,
			XMapException, UnexpectedException, XServiceException,
			MalformedURLException, HeadlessException {

		// Step 4.1: Create imageInfo and set the frame size and image format.
		ImageInfo imageInfo = new ImageInfo();
		imageInfo.setFormat(ImageFileFormat.GIF);
		imageInfo.setHeight(600);
		imageInfo.setWidth(800);
		imageInfo.setImageParameter("");

		// Step 4.2: Build mapParams
		MapParams mapParams = new MapParams();
		mapParams.setShowScale(true);
		mapParams.setUseMiles(true);
		mapParams.setLanguage("x-ptv-DFT"); //optional
		
		// Step 4.3: Set argument includeImageInResponse to false (default).
		boolean includeImageInResponse = false;

		// Step 4.4: Create a line from the calculated route
		LineString[] lineStrings = new LineString[] { route.getPolygon() };
		Lines[] lines = new Lines[1];
		LineOptions options = new LineOptions();
		LinePartOptions partoptions = new LinePartOptions();
		partoptions.setColor(new Color(0, 0, 255));
		partoptions.setVisible(true);
		partoptions.setWidth(-10);
		options.setMainLine(partoptions);

		lines[0] = new Lines(lineStrings, options);

		// Step 4.5: Define customLayer that contains the object lines and set layers.
		CustomLayer customLayer = new CustomLayer();
		customLayer.setVisible(true);
		customLayer.setDrawPriority(100); // this is PRIORROUTE
		customLayer.setLines(lines);
		customLayer.setObjectInfos(ObjectInfoType.NONE);
		customLayer.setCenterObjects(true);
		Layer[] layers = new Layer[] { customLayer };
		
		// Step 4.6: Set boundingBox fand use corners from the calculated route
		BoundingBox boundingBox = new BoundingBox();
		boundingBox.setLeftTop(route.getTotalRectangle().getLeftBottom());
		boundingBox.setRightBottom(route.getTotalRectangle().getRightTop());

		// Step 4.7: Return object map using the following method.
		Map map = xMapClient.renderMapBoundingBox(boundingBox, mapParams,
				imageInfo, layers, includeImageInResponse);

		// Step 4.8: Retrieve the image
		String imageUrl = "http://" + map.getImage().getUrl();
		ImageIcon icon = new ImageIcon(new URL(imageUrl));

		// Step 4.9: Display image using JFrame.
		JFrame frame = new JFrame("Sample Map");
		frame.setSize(icon.getIconWidth() + 10, icon.getIconHeight() + 30);
		frame.getContentPane().add(new JLabel(icon));
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	/**
	 * Returns a list of pre-configured waypoints.
	 * 
	 * @return list of WaypointDesc suitable for routing
	 */
	private static WaypointDesc[] getWaypoints() {
		// Step 3.3.1: Set points = list of pairs of (x,y) coordinates for start and destination
		double[][] points = new double[][] { { 678730.0, 6372820.0 },
				{ 683080.0, 6372430.0 } };

		// Step 3.3.2: Define waypointDescs, set points and link type as next segment
		WaypointDesc[] waypointDescs = new WaypointDesc[points.length];
		for (int i = 0; i < points.length; i++) {
			Point pt = new Point();
			pt.setPoint(new PlainPoint(points[i][0], points[i][1], null));

			waypointDescs[i] = new WaypointDesc();
			waypointDescs[i].setCoords(new Point[] { pt });
			waypointDescs[i].setLinkType(LinkType.NEXT_SEGMENT);
		}
		return waypointDescs;
	}

}
